//
// Project : (n)curses widget library
// File    : app.cc
// Purpose : class for the global application
//

#include <stdio.h>

#include "LCD.hh"

#include "Application.hh"

namespace LWL
{

//
// defines
//

#define MAX_QUEUE_SIZE   1000
#define MAX_HANDLE_NO    5

#define MAX_SLEEP_TIMEOUT  5000

//
// global application object
//

Application * global_app = NULL;

//
// constructor and destructor
//

Application::Application ()
{
    _end = false;

    if ( global_app != NULL )
    {
        printf( "(Application) constructor : can not have more than one application\n" );
        exit( 1 );
    }// if

    global_app = this;
    
    init();
}

Application::~Application ()
{
    delete _desktop;
}

//
// main loop and event handling
//

void
Application::run ()
{
    int  i;
    
    _desktop->draw();
    LCD::flush();
    
    while ( ! _end )
    {
        // check for key input
        check_key();

        // handle all pending events
        i = 0;
        while ( _event_queue.size() > 0 )
        {
            Event  ev = _event_queue.behead();

            // handle events
            _desktop->handle_event( ev );

            // limit number of handled events
            if ( i++ > MAX_HANDLE_NO )
                break;
        }// while
        
        // draw desktop
        _desktop->draw();
        LCD::flush();
    }// while

    LCD::clear();
    LCD::invalid();
    LCD::flush();
}

//
// add event to queue
//
void
Application::add_event ( Event & e )
{
    if ( _event_queue.size() < MAX_QUEUE_SIZE )
        _event_queue.append( e );
    else
        printf( "(Application) add_event : queue reached maximal size\n" );
}

//
// check for new key events
//
void
Application::check_key ()
{
    char   buf[256];
    Event  ev;
    
    if ( LCD::read( buf, sizeof( buf ), MAX_SLEEP_TIMEOUT ) > 0 )
    {
        if ( buf[0] == 'b' )
        {
            ev.set_keycode( buf[2] );
            add_event( ev );
        }// if
    }// if
}  

//
// set up ncurses
//
void
Application::init ()
{
    LCD::init( 20, 4, "/dev/usb/tts/0" );
    LCD::clear();
    LCD::invalid();
    
    _desktop = new Desktop( LCD::width(), LCD::height() );
}

}// namespace
