#ifndef __APP_HH
#define __APP_HH
//
// Project : LCD widget library
// File    : app.hh
// Purpose : class for the global application
//

#include "Window.hh"
#include "Desktop.hh"
#include "SLL.hh"

namespace LWL
{

//
// manages windows and handles events
//
class Application
{
protected:
    // application window
    Desktop    *  _desktop;

    // indicate end of app
    bool          _end;

    // event queue
    SLL< Event >  _event_queue;
    
public:
    //
    // constructor and destructor
    //

    Application ();

    virtual ~Application ();

    //
    // main loop and event handling
    //

    virtual void run ();

    virtual void end () { _end = true; }

    // add event to queue
    virtual void add_event ( Event & e );

    // check for key event
    virtual void check_key ();
    
    //
    // manage windows
    //

    virtual Desktop       * desktop ()       { return _desktop; }
    virtual const Desktop * desktop () const { return _desktop; }
    
    virtual void add_window ( Window * win ) { _desktop->add_window( win ); }
    
protected:
    // setup ncurses
    void init ();
};

// global application object
extern Application  * global_app;

}// namespace

#endif  // __APP_HH
