//
//  Project   : misc
//  File      : DLL.cc
//  Author    : Ronald Kriemann
//  Purpose   : Template class for a double concatenated list
//

template <class T>
DLL<T> &
DLL<T>::prepend ( const T & elem )
{
    TItem * tmp = new TItem( elem, _first );

    if (_first)
        _first->_prev = tmp;
    
    if (_last == NULL)
        _last = tmp;
    
    _first = tmp;
    _size++;
    
    return *this;
}

template <class T>
DLL<T> &
DLL<T>::append ( const T & elem )
{
    TItem * tmp = new TItem( elem, NULL, _last );
    
    if (_last)
        _last->_next = tmp;
    
    _last = tmp;
    
    if ( _first == NULL )
        _first = tmp;
    
    _size++;
    
    return *this;
}

template <class T>
void
DLL<T>::remove ( const T & elem )
{
    TItem  * item = _first;
    TItem  * old  = NULL;

    while (item != NULL)
    {
        if (item->_data == elem)
        {
            if (item == _first)
            {
                _first = item->_next;

                if ( _first == NULL )
                    _last = NULL;
                else
                    _first->_prev = NULL;
            }// if
            else // (item != first) => (old != NULL)
            {
                old->_next = item->_next;
                item->_next->_prev = old;
            }// else

            delete item;
            _size--;

            if ( old == NULL )
                item = _first;
            else
                item = old->_next;
        }// if
        else
        {
            old  = item;
            item = item->_next;
        }// else
    }// while
}

template <class T>
void
DLL<T>::remove_first ()
{
    if (_first)
    {
        TItem  * tmp = _first->_next;

        delete _first;

        if (tmp == NULL)
            _last = NULL;
        else
            tmp->_prev = NULL;

        _first = tmp;

        _size--;
    }// 
}

template <class T>
void
DLL<T>::remove_last ()
{
    if (_last)
    {
        TItem  * tmp = _last->_prev;

        delete _last;

        if (tmp == NULL)
            _first = NULL;
        else
            tmp->_next = NULL;

        _last = tmp;

        _size--;
    }// 
}

template <class T>
T
DLL<T>::behead ()
{
    assert( _first != NULL );

    T        ret = _first->_data;
    TItem  * tmp = _first->_next;

    delete _first;

    if (tmp == NULL)
        _last = NULL;
    else
        tmp->_prev = NULL;
    
    _first = tmp;
    
    _size--;
    
    return ret;
}

template <class T>
void
DLL<T>::remove_all ()
{
    if (_size == 0)
        return;
    
    TItem * tmp = _first;
    TItem * next;
    
    while (tmp != NULL)
    {
        next = tmp->_next;
        delete tmp;
        tmp = next;
    }// while
    
    _first = _last = NULL;
    _size = 0;
}

template <class T>
void
DLL<T>::copy ( const DLL< T > & list )
{
    TItem  * tmp  = list._first;
    TItem  * old  = NULL;
    TItem  * item = list._first;

    remove_all();
    
    while ( tmp != NULL )
    {
        item = new TItem( tmp->_data );

        if ( _first == NULL )
            _first = item;

        item->prev = old;
        
        if ( old != NULL )
            old->_next = item;

        old = item;
        tmp = tmp->_next;
    }// while

    _last = old;
    _size = list._size;
}
    
template <class T>
bool
DLL<T>::is_in ( const T & elem )
{
    TItem * tmp = _first;

    while ( tmp )
    {
        if (tmp->_data == elem)
            return true;

        tmp = tmp->_next;
    }// while

    return false;
}

/////////////////////////////////////////////////
//
// misc.
//

//
// return size in bytes used by this object
//
template <class T>
unsigned long
DLL<T>::byte_size () const
{
    return sizeof(TItem*) + sizeof(TItem*) + sizeof(unsigned int) + (_size * sizeof(TItem));
}
