//
//  Project   : misc
//  File      : SLL.cc
//  Author    : Ronald Kriemann
//  Purpose   : Template class for a single concatenated list
//

template <class T>
SLL<T> &
SLL<T>::prepend ( const T & elem )
{
    Item  * tmp = new Item( elem, _first );
    
    _first = tmp;
    _size++;

    if (_last == NULL)
        _last = tmp;
    
    return *this;
}

template <class T>
SLL<T> &
SLL<T>::append ( const T & elem )
{
    Item * tmp = new Item( elem );
    
    if (_last)
        _last->_next = tmp;
    
    _last = tmp;
    
    if ( ! _first )
        _first = tmp;
    
    _size++;
    
    return *this;
}

template <class T>
void
SLL<T>::remove ( const T & elem )
{
    Item  * item = _first;
    Item  * old  = NULL;

    while (item != NULL)
    {
        if (item->_data == elem)
        {
            if (item == _first)
                _first = item->_next;
            else // (item != first) => (old != NULL)
                old->_next = item->_next;

            if (item == _last)
                _last = old;
            
            delete item;
            _size--;

            if ( old == NULL )
                item = _first;
            else
                item = old->_next;
        }// if
        else
        {
            old  = item;
            item = item->_next;
        }// else
    }// while
}

template <class T>
void
SLL<T>::remove_first ()
{
    if (_first)
    {
        Item  * tmp = _first->_next;

        delete _first;

        if (tmp == NULL)
            _last = NULL;

        _first = tmp;

        _size--;
    }// if
}

template <class T>
T
SLL<T>::behead ()
{
    assert( _first != NULL );

    T        ret = _first->_data;
    Item  * tmp = _first->_next;

    delete _first;

    if (tmp == NULL)
        _last = NULL;
    
    _first = tmp;
    
    _size--;
    
    return ret;
}

template <class T>
void
SLL<T>::remove_all ()
{
    if (_size == 0)
        return;
    
    Item * tmp = _first;
    Item * next;
    
    while (tmp != NULL)
    {
        next = tmp->_next;
        delete tmp;
        tmp = next;
    }// while
    
    _first = _last = NULL;
    _size = 0;
}

template <class T>
void
SLL<T>::copy ( const SLL< T > & list )
{
    Item  * tmp  = list._first;
    Item  * old  = NULL;
    Item  * item = list._first;

    remove_all();
    
    while ( tmp != NULL )
    {
        item = new Item( tmp->_data );

        if ( _first == NULL )
            _first = item;
        
        if ( old != NULL )
            old->_next = item;

        old = item;
        tmp = tmp->_next;
    }// while

    _last = old;
    _size = list._size;
}
    
template <class T>
bool
SLL<T>::is_in ( const T & elem )
{
    Item * tmp = _first;

    while ( tmp )
    {
        if (tmp->_data == elem)
            return true;

        tmp = tmp->_next;
    }// while

    return false;
}

/////////////////////////////////////////////////
//
// misc.
//

//
// return size in bytes used by this object
//
template <class T>
unsigned long
SLL<T>::byte_size () const
{
    return sizeof(Item*) + sizeof(Item*) + sizeof(unsigned int) + (_size * sizeof(Item));
}
    
