//
// Project : (n)curses widget library
// File    : main.cc
// Purpose : demo program
//

#include <stdio.h>

#include "Application.hh"
#include "LCD.hh"
#include "Window.hh"
#include "Menu.hh"

using namespace LWL;

/////////////////////////////////////////////
//
// defines
//
/////////////////////////////////////////////

#define CMD_MENU_SYS_LOAD  0x101
#define CMD_MENU_SYS_MEM   0x102
#define CMD_MENU_SYS_SWAP  0x103
#define CMD_MENU_SYS_UP    0x104

#define CMD_MENU_CONF_TIMEOUT  0x201
#define CMD_MENU_CONF_LED      0x202
#define CMD_MENU_CONF_VERSION  0x203

class MyWindow : public Window
{
protected:
    int _pos;
    unsigned char _fill;
    
public:
    MyWindow ( int x, int y, int w, int h )
            : Window( x, y, w, h ), _pos( 0 ), _fill( 120 )
    {}

    virtual void handle_event ( Event & e )
    {
        if ( e.type() == Event::EV_KEYBOARD )
        {
            switch ( e.keycode() )
            {
            case BTN_MIDDLE : _fill++; e.clear(); break;
            case BTN_RIGHT  : global_app->end(); e.clear(); break;
            }
        }// if
        else if ( e.type() == Event::EV_COMMAND )
        {
            switch ( e.command() )
            {
            case CMD_MENU_SYS_LOAD : _fill = 'C'; e.clear(); break;
            case CMD_MENU_SYS_MEM  : _fill = 'M'; e.clear(); break;
            case CMD_MENU_SYS_SWAP : _fill = 'S'; e.clear(); break;
            case CMD_MENU_SYS_UP   : _fill = 'U'; e.clear(); break;

            case CMD_MENU_CONF_TIMEOUT : _fill = 'T'; e.clear(); break;
            case CMD_MENU_CONF_LED     : _fill = 'L'; e.clear(); break;
            case CMD_MENU_CONF_VERSION : _fill = 'V'; e.clear(); break;
            }// switch
        }// if
    }
    
    virtual void draw ()
    {
        fill( _fill );

        char  buf[16];
        
        snprintf( buf, sizeof(buf), " 0x%02x ", _fill );

        print_str( 1, 0, buf );
//         print_str( _pos, 0, "R" );
//         print_str( _pos, 1, "K" );

//         _pos++;
        
//         if ( _pos+1 > width() )
//             _pos = 0;
    }
};

int
main ( int argc, char ** argv )
{
    Application  * app = new Application;
    Menu         * menu = new Menu( 0, 0, 10, 4 );
    Menu::Entry  * sub;

    sub = new Menu::Entry( "System" );
    sub->add_submenu( new Menu::Entry( "Load", CMD_MENU_SYS_LOAD ) );
    sub->add_submenu( new Menu::Entry( "Mem", CMD_MENU_SYS_MEM ) );
    sub->add_submenu( new Menu::Entry( "Swap", CMD_MENU_SYS_SWAP ) );
    sub->add_submenu( new Menu::Entry( "Uptime", CMD_MENU_SYS_UP ) );
    menu->add_entry( sub );
    
    sub = new Menu::Entry( "Config" );
    sub->add_submenu( new Menu::Entry( "Update", CMD_MENU_CONF_TIMEOUT ) );
    sub->add_submenu( new Menu::Entry( "LED", CMD_MENU_CONF_LED ) );
    sub->add_submenu( new Menu::Entry( "Version", CMD_MENU_CONF_VERSION ) );
    menu->add_entry( sub );
    
    app->add_window( new MyWindow( 11, 1, 8, 2 ) );
    app->add_window( menu );
    
    app->run();

    return 0;
}
